"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const widgets_1 = require("@phosphor/widgets");
const apputils_1 = require("@jupyterlab/apputils");
const CredentialsWidget_1 = require("./CredentialsWidget");
// The class name added to the extension, this class ensures the background to be white
const CLASS_NAME = 'jp-FileBrowser';
class CredentialsPanel extends widgets_1.Widget {
    constructor(options) {
        super();
        this.serviceManager = options.serviceManager;
        this.id = options.id;
        this.addClass(CLASS_NAME);
        this.toolbar = new apputils_1.Toolbar();
        this.setSaveListener = this.setSaveListener.bind(this);
        let saveButton = new apputils_1.ToolbarButton({
            iconClassName: 'jp-SaveIcon jp-Icon jp-Icon-16',
            tooltip: 'Save',
            onClick: () => this.onSave()
        });
        this.setAddCredentialListener = this.setAddCredentialListener.bind(this);
        let newCredential = new apputils_1.ToolbarButton({
            iconClassName: 'jp-AddIcon jp-Icon jp-Icon-16',
            tooltip: 'New Credential',
            onClick: () => this.onAddCredential()
        });
        this.setLoginListener = this.setLoginListener.bind(this);
        let loginButton = new apputils_1.ToolbarButton({
            iconClassName: 'jp-KeyIcon jp-Icon jp-Icon-16',
            tooltip: 'Login',
            onClick: () => {
                this.onLogin();
                this.layout.removeWidget(this.toolbar);
                this.toolbar = new apputils_1.Toolbar();
                this.toolbar.addItem('stopButton', stopButton);
                this.toolbar.addItem('newCredential', newCredential);
                this.toolbar.addItem('saveButton', saveButton);
                layout.insertWidget(0, this.toolbar);
            }
        });
        this.toolbar.addItem('loginButton', loginButton);
        this.setStopListener = this.setStopListener.bind(this);
        let stopButton = new apputils_1.ToolbarButton({
            iconClassName: 'jp-StopIcon jp-Icon jp-Icon-16',
            tooltip: 'Stop',
            onClick: () => {
                this.onStop();
                this.layout.removeWidget(this.toolbar);
                this.toolbar = new apputils_1.Toolbar();
                this.toolbar.addItem('loginButton', loginButton);
                layout.insertWidget(0, this.toolbar);
            }
        });
        let layout = new widgets_1.PanelLayout();
        layout.addWidget(this.toolbar);
        layout.addWidget(new CredentialsWidget_1.CredentialsWidget({
            serviceManager: options.serviceManager,
            setSaveListener: this.setSaveListener,
            setAddCredentialListener: this.setAddCredentialListener,
            setLoginListener: this.setLoginListener,
            setStopListener: this.setStopListener
        }));
        this.layout = layout;
    }
    setAddCredentialListener(onAddCredential) {
        this.onAddCredential = onAddCredential;
    }
    setSaveListener(onSave) {
        this.onSave = onSave;
    }
    setLoginListener(onLogin) {
        this.onLogin = onLogin;
    }
    setStopListener(onStop) {
        this.onStop = onStop;
    }
}
exports.CredentialsPanel = CredentialsPanel;
