"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const rootselector_1 = require("./rootselector");
const reselect_1 = require("reselect");
// Actions
function setCredential(id, tag, value, changed) {
    return {
        type: 'credential/set',
        payload: { id: id, tag: tag, value: value, changed: changed }
    };
}
exports.setCredential = setCredential;
function addCredential() {
    return {
        type: 'credential/create',
        payload: undefined
    };
}
exports.addCredential = addCredential;
function removeCredential(id) {
    return {
        type: 'credential/destroy',
        payload: id
    };
}
exports.removeCredential = removeCredential;
function setLastId(lastId) {
    return {
        type: 'lastid/set',
        payload: lastId
    };
}
exports.setLastId = setLastId;
// the variable name must match the name of the reducer!!!
// returns the dict that contains the data of this reducer
exports.getCredentialsRoot = reselect_1.createSelector(rootselector_1.getRoot, ({ credentialsReducer }) => credentialsReducer);
exports.getCredentials = reselect_1.createSelector(exports.getCredentialsRoot, ({ credentials }) => credentials);
exports.getLastId = reselect_1.createSelector(exports.getCredentialsRoot, ({ lastId }) => lastId);
// Reducer
const defaultState = {
    lastId: 0,
    credentials: []
};
function pad(num, size) {
    var s = num + "";
    while (s.length < size)
        s = "0" + s;
    return s;
}
function credentialsReducer(state = defaultState, { type, payload }) {
    switch (type) {
        case 'credential/destroy':
            return Object.assign({}, state, { credentials: state.credentials.filter(c => c.id !== payload) });
        case 'credential/create':
            let { lastId } = state;
            lastId = lastId + 1;
            return Object.assign({}, state, { lastId, credentials: [
                    ...state.credentials,
                    { id: 'id_' + pad(lastId, 6), tag: undefined, value: undefined, changed: true }
                ].sort((a, b) => a.id > b.id ? 1 : -1) });
        case 'credential/set':
            return Object.assign({}, state, { credentials: [
                    ...state.credentials.filter(c => c.id !== payload.id),
                    payload
                ].sort((a, b) => a.id > b.id ? 1 : -1) });
        case 'lastid/set':
            return Object.assign({}, state, { lastId: payload });
        default:
            return state;
    }
}
exports.default = credentialsReducer;
